@@ -1,12 +1,16 @@
 import argparse
 import configargparse
-import functools
 import inspect
 import logging
 import sys
 import tempfile
 import types
 
+try:
+    import mock
+except ImportError:
+    from unittest import mock
+
 if sys.version_info < (2, 7):
     import unittest2 as unittest
 else:
@@ -270,6 +274,16 @@ class TestBasicUseCases(TestCase):
     def testBasicCase2_WithGroups(self):
         self.testBasicCase2(use_groups=True)
 
+    def testPositionalAndConfigVarLists(self):
+        self.initParser()
+        self.add_arg("a")
+        self.add_arg("-x", "--arg", nargs="+")
+
+        ns = self.parse("positional_value", config_file_contents="""arg = [Shell, someword, anotherword]""")
+
+        self.assertEqual(ns.arg, ['Shell', 'someword', 'anotherword'])
+        self.assertEqual(ns.a, "positional_value")
+
 
     def testMutuallyExclusiveArgs(self):
         config_file = tempfile.NamedTemporaryFile(mode="w", delete=True)
@@ -455,25 +469,60 @@ class TestBasicUseCases(TestCase):
         self.initParser(ignore_unknown_config_file_keys=False)
         ns, args = self.parse_known(args="-x 1", config_file_contents="bla=3",
             env_vars={"bla": "2"})
-        self.assertListEqual(args, ["--bla", "3", "-x", "1"])
+        self.assertEqual(set(args), set(["--bla", "3", "-x", "1"]))
+
+    def testBooleanValuesCanBeExpressedAsNumbers(self):
+        self.initParser()
+        store_true_env_var_name = "STORE_TRUE"
+        self.add_arg("--boolean_store_true", action="store_true", env_var=store_true_env_var_name)
+
+        result_namespace = self.parse("", config_file_contents="""boolean_store_true = 1""")
+        self.assertTrue(result_namespace.boolean_store_true)
+
+        result_namespace = self.parse("", config_file_contents="""boolean_store_true = 0""")
+        self.assertFalse(result_namespace.boolean_store_true)
+
+        result_namespace = self.parse("", env_vars={store_true_env_var_name: "1"})
+        self.assertTrue(result_namespace.boolean_store_true)
+
+        result_namespace = self.parse("", env_vars={store_true_env_var_name: "0"})
+        self.assertFalse(result_namespace.boolean_store_true)
+
+        self.initParser()
+        store_false_env_var_name = "STORE_FALSE"
+        self.add_arg("--boolean_store_false", action="store_false", env_var=store_false_env_var_name)
+
+        result_namespace = self.parse("", config_file_contents="""boolean_store_false = 1""")
+        self.assertFalse(result_namespace.boolean_store_false)
+
+        result_namespace = self.parse("", config_file_contents="""boolean_store_false = 0""")
+        self.assertTrue(result_namespace.boolean_store_false)
+
+        result_namespace = self.parse("", env_vars={store_false_env_var_name: "1"})
+        self.assertFalse(result_namespace.boolean_store_false)
+
+        result_namespace = self.parse("", env_vars={store_false_env_var_name: "0"})
+        self.assertTrue(result_namespace.boolean_store_false)
 
     def testConfigOrEnvValueErrors(self):
         # error should occur when a flag arg is set to something other than "true" or "false"
         self.initParser()
         self.add_arg("--height", env_var = "HEIGHT", required=True)
         self.add_arg("--do-it", dest="x", env_var = "FLAG1", action="store_true")
-        self.add_arg("--dont-do-it", dest="x", env_var = "FLAG2", action="store_false")
+        self.add_arg("--dont-do-it", dest="y", env_var = "FLAG2", action="store_false")
         ns = self.parse("", env_vars = {"HEIGHT": "tall", "FLAG1": "yes"})
         self.assertEqual(ns.height, "tall")
         self.assertEqual(ns.x, True)
+        ns = self.parse("", env_vars = {"HEIGHT": "tall", "FLAG2": "yes"})
+        self.assertEqual(ns.y, False)
         ns = self.parse("", env_vars = {"HEIGHT": "tall", "FLAG2": "no"})
-        self.assertEqual(ns.x, False)
+        self.assertEqual(ns.y, True)
 
         # error should occur when flag arg is given a value
         self.initParser()
         self.add_arg("-v", "--verbose", env_var="VERBOSE", action="store_true")
         self.assertParseArgsRaises("Unexpected value for VERBOSE: 'bla'. "
-                                   "Expecting 'true', 'false', 'yes', or 'no'",
+                                   "Expecting 'true', 'false', 'yes', 'no', '1' or '0'",
             env_vars={"VERBOSE" : "bla"})
         ns = self.parse("",
                         config_file_contents="verbose=true",
@@ -485,6 +534,8 @@ class TestBasicUseCases(TestCase):
         self.assertEqual(ns.verbose, True)
         ns = self.parse("", env_vars = {"HEIGHT": "true", "VERBOSE": "true"})
         self.assertEqual(ns.verbose, True)
+        ns = self.parse("", env_vars = {"HEIGHT": "true", "VERBOSE": "false"})
+        self.assertEqual(ns.verbose, False)
         ns = self.parse("", config_file_contents="--verbose",
                         env_vars = {"HEIGHT": "true"})
         self.assertEqual(ns.verbose, True)
@@ -516,6 +567,38 @@ class TestBasicUseCases(TestCase):
         self.assertEqual(ns.arg4, "arg4_value")
         self.assertEqual(ns.arg4_more, "magic")
 
+    def testEnvVarLists(self):
+        self.initParser()
+        self.add_arg("-x", "--arg2", env_var="TEST2")
+        self.add_arg("-y", "--arg3", env_var="TEST3", type=int)
+        self.add_arg("-z", "--arg4", env_var="TEST4", nargs="+")
+        self.add_arg("-u", "--arg5", env_var="TEST5", nargs="+", type=int)
+        self.add_arg("--arg6", env_var="TEST6")
+        self.add_arg("--arg7", env_var="TEST7", action="append")
+        ns = self.parse("", env_vars={"TEST2": "22",
+                                      "TEST3": "22",
+                                      "TEST4": "[Shell, someword, anotherword]",
+                                      "TEST5": "[22, 99, 33]",
+                                      "TEST6": "[value6.1, value6.2, value6.3]",
+                                      "TEST7": "[value7.1, value7.2, value7.3]",
+                                      })
+        self.assertEqual(ns.arg2, "22")
+        self.assertEqual(ns.arg3, 22)
+        self.assertEqual(ns.arg4, ['Shell', 'someword', 'anotherword'])
+        self.assertEqual(ns.arg5, [22, 99, 33])
+        self.assertEqual(ns.arg6, "[value6.1, value6.2, value6.3]")
+        self.assertEqual(ns.arg7, ["value7.1", "value7.2", "value7.3"])
+
+    def testPositionalAndEnvVarLists(self):
+        self.initParser()
+        self.add_arg("a")
+        self.add_arg("-x", "--arg", env_var="TEST", nargs="+")
+
+        ns = self.parse("positional_value", env_vars={"TEST": "[Shell, someword, anotherword]"})
+
+        self.assertEqual(ns.arg, ['Shell', 'someword', 'anotherword'])
+        self.assertEqual(ns.a, "positional_value")
+
 class TestMisc(TestCase):
     # TODO test different action types with config file, env var
 
@@ -523,6 +606,14 @@ class TestMisc(TestCase):
     def setUp(self):
         self.initParser(args_for_setting_config_path=[])
 
+    @mock.patch('argparse.ArgumentParser.__init__')
+    def testKwrgsArePassedToArgParse(self, argparse_init):
+        kwargs_for_argparse = {"allow_abbrev": False, "whatever_other_arg": "something"}
+
+        parser = configargparse.ArgumentParser(add_config_file_help=False, **kwargs_for_argparse)
+
+        argparse_init.assert_called_with(parser, **kwargs_for_argparse)
+
     def testGlobalInstances(self, name=None):
         p = configargparse.getArgumentParser(name, prog="prog", usage="test")
         self.assertEqual(p.usage, "test")
@@ -612,6 +703,27 @@ class TestMisc(TestCase):
         ns, args = self.parse_known(config_file_contents="arg1 = 3")
         self.assertEqual(getattr(ns, "arg1", ""), "")
 
+    def test_AbbrevConfigFileArgs(self):
+        """Tests that abbreviated values don't get pulled from config file.
+
+        """
+        temp_cfg = tempfile.NamedTemporaryFile(mode="w", delete=True)
+        temp_cfg.write("a2a = 0.5\n")
+        temp_cfg.write("a3a = 0.5\n")
+        temp_cfg.flush()
+
+        self.initParser()
+
+        self.add_arg('-c', '--config_file', required=False, is_config_file=True,
+                     help='config file path')
+
+        self.add_arg('--hello', type=int, required=False)
+
+        command = '-c {0} --hello 2'.format(temp_cfg.name)
+
+        known, unknown = self.parse_known(command)
+
+        self.assertListEqual(unknown, ['--a2a', '0.5', '--a3a', '0.5'])
 
     def test_FormatHelp(self):
         self.initParser(args_for_setting_config_path=["-c", "--config"],
@@ -643,6 +755,15 @@ class TestMisc(TestCase):
             '--flag \s*Flag help text'
         )
 
+    class CustomClass(object):
+        def __init__(self, name):
+            self.name = name
+
+    @staticmethod
+    def valid_custom(s):
+        if s == "invalid": raise Exception("invalid name")
+        return TestMisc.CustomClass(s)
+
     def testConstructor_WriteOutConfigFileArgs(self):
         # Test constructor args:
         #   args_for_writing_out_config_file
@@ -656,12 +777,14 @@ class TestMisc(TestCase):
         self.add_arg("--config-file-settable-arg", type=int)
         self.add_arg("--config-file-settable-arg2", type=int, default=3)
         self.add_arg("--config-file-settable-flag", action="store_true")
+        self.add_arg("--config-file-settable-custom", type=TestMisc.valid_custom)
         self.add_arg("-l", "--config-file-settable-list", action="append")
 
         # write out a config file
         command_line_args = "-w %s " % cfg_f.name
         command_line_args += "--config-file-settable-arg 1 "
         command_line_args += "--config-file-settable-flag "
+        command_line_args += "--config-file-settable-custom custom_value "
         command_line_args += "-l a -l b -l c -l d "
 
         self.assertFalse(self.parser._exit_method_called)
@@ -672,6 +795,7 @@ class TestMisc(TestCase):
         cfg_f.seek(0)
         expected_config_file_contents = "config-file-settable-arg = 1\n"
         expected_config_file_contents += "config-file-settable-flag = true\n"
+        expected_config_file_contents += "config-file-settable-custom = custom_value\n"
         expected_config_file_contents += "config-file-settable-list = [a, b, c, d]\n"
         expected_config_file_contents += "config-file-settable-arg2 = 3\n"
 
@@ -724,6 +848,49 @@ class TestMisc(TestCase):
         self.assertRaisesRegex(ValueError, "Couldn't open / for writing:",
                                 self.parse, args = command_line_args + " -w /")
 
+    def testConstructor_WriteOutConfigFileArgsLong(self):
+        """Test config writing with long version of arg
+
+        There was a bug where the long version of the
+        args_for_writing_out_config_file was being dumped into the resultant
+        output config file
+        """
+        # Test constructor args:
+        #   args_for_writing_out_config_file
+        #   write_out_config_file_arg_help_message
+        cfg_f = tempfile.NamedTemporaryFile(mode="w+", delete=True)
+        self.initParser(args_for_writing_out_config_file=["--write-config"],
+                        write_out_config_file_arg_help_message="write config")
+
+
+        self.add_arg("-not-config-file-settable")
+        self.add_arg("--config-file-settable-arg", type=int)
+        self.add_arg("--config-file-settable-arg2", type=int, default=3)
+        self.add_arg("--config-file-settable-flag", action="store_true")
+        self.add_arg("-l", "--config-file-settable-list", action="append")
+
+        # write out a config file
+        command_line_args = "--write-config %s " % cfg_f.name
+        command_line_args += "--config-file-settable-arg 1 "
+        command_line_args += "--config-file-settable-flag "
+        command_line_args += "-l a -l b -l c -l d "
+
+        self.assertFalse(self.parser._exit_method_called)
+
+        ns = self.parse(command_line_args)
+        self.assertTrue(self.parser._exit_method_called)
+
+        cfg_f.seek(0)
+        expected_config_file_contents = "config-file-settable-arg = 1\n"
+        expected_config_file_contents += "config-file-settable-flag = true\n"
+        expected_config_file_contents += "config-file-settable-list = [a, b, c, d]\n"
+        expected_config_file_contents += "config-file-settable-arg2 = 3\n"
+
+        self.assertEqual(cfg_f.read().strip(),
+            expected_config_file_contents.strip())
+        self.assertRaisesRegex(ValueError, "Couldn't open / for writing:",
+            self.parse, args = command_line_args + " --write-config /")
+
     def testMethodAliases(self):
         p = self.parser
         p.add("-a", "--arg-a", default=3)
